# /*******************************************************************************
#  * Copyright 2016 -- 2022 IBM Corporation
#  *
#  * Licensed under the Apache License, Version 2.0 (the "License");
#  * you may not use this file except in compliance with the License.
#  * You may obtain a copy of the License at
#  *
#  *     http://www.apache.org/licenses/LICENSE-2.0
#  *
#  * Unless required by applicable law or agreed to in writing, software
#  * distributed under the License is distributed on an "AS IS" BASIS,
#  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  * See the License for the specific language governing permissions and
#  * limitations under the License.
# *******************************************************************************/

#  *
#  *                       cloudFPGA
#  *     Copyright IBM Research, All Rights Reserved
#  *    =============================================
#  *     Created: Mar. 2021
#  *     Authors: FAB, WEI, NGL, DID
#  *
#  *     Description:
#  *      A Python library with functions for accessing cFRM and create/delete images, instances, clusters etc.
#  *
#  *

import os
import sys
import requests
from requests.utils import requote_uri
import json
import time
import datetime

from cfsp_user import *
from cfsp_globals import *

from util import errorReqExit




####################################################################################################
# Clusters functions
####################################################################################################


class cFcluster:
    """Representation of a specific cloudFPGA cluster"""

    def __init__(self, user: cFuser, cluster_data):
        self.user = user
        self.cluster_data = cluster_data
        self.id = cluster_data['cluster_id']

    def get_id(self):
        return self.id


def post_cluster(user: cFuser, number_of_FPGA_nodes, role_image_id, host_address):
    # build cluster_req structure
    print("Creating FPGA cluster...")
    try:
        start = time.time()
        sw_rank = 0
        cluster_req = []
        rank0node = {'image_id': __NON_FPGA_IDENTIFIER__,
                     'node_id': sw_rank,
                     'node_ip': host_address}
        cluster_req.append(rank0node)
        size = number_of_FPGA_nodes + 1
        for i in range(1, size):
            fpgaNode = {
                'image_id': str(role_image_id),
                'node_id': i
            }
            cluster_req.append(fpgaNode)

        r1 = requests.post(
            "http://" + __cf_manager_url__ + "/clusters?{0}&dont_verify_memory=0".format(
                user.get_auth_string(with_project=True)),
            json=cluster_req, timeout=__POST_CLUSTER_TIMEOUT__)
        elapsed = time.time() - start

        if r1.status_code != 200:
            # something went wrong
            return errorReqExit("POST cluster", r1.status_code)

        cluster_data = json.loads(r1.text)
        print("Id of new cluster: {}".format(cluster_data['cluster_id']))
        print("Time for POST cluster: \t{0}s\n".format(elapsed))
        new_cluster = cFcluster(user, cluster_data)
        return new_cluster
    except requests.exceptions.Timeout as e:
        # Maybe set up for a retry
        print(e)
        print("ERROR: Something went wrong with post_cluster request and it reached timeout="+str(__POST_CLUSTER_TIMEOUT__)+". Maybe retry or increase timeout value.\n")
        sys.exit(1)


def get_cluster_data(cluster: cFcluster):
    print("Requesting cluster data for cluster_id={0} ...".format(cluster.get_id())) # FIXME: .get_id()
    try:
        start = time.time()
        r1 = requests.get(
            "http://" + __cf_manager_url__ + "/clusters/" + str(cluster.get_id()) + "?{0}".format(
                cluster.user.get_auth_string()), timeout=__GET_CLUSTER_TIMEOUT__)
        elapsed = time.time() - start
        print("Time for GET cluster: \t{0}s\n".format(elapsed))
        if r1.status_code != 200:
            # something went horrible wrong
            return errorReqExit("GET cluster", r1.status_code)

        cluster_data = json.loads(r1.text)
        # update, if necessary
        cluster.cluster_data = cluster_data
        return cluster_data
    except requests.exceptions.Timeout as e:
        # Maybe set up for a retry
        print(e)
        print("ERROR: Something went wrong with get_cluster_data request and it reached timeout="+str(__GET_CLUSTER_TIMEOUT__)+". Maybe retry or increase timeout value.\n")


def get_clusters_data(user: cFuser, limit=100):
    print("Requesting clusters data (limit="+str(limit)+")...")
    try:
        start = time.time()
        r1 = requests.get(
            "http://" + __cf_manager_url__ + "/clusters" + "?{0}&limit={1}".format(
                user.get_auth_string(), limit), timeout=__GET_CLUSTER_TIMEOUT__)
        elapsed = time.time() - start
        print(r1.request.url)
        print("Time for GET clusters: \t{0}s\n".format(elapsed))
        if r1.status_code != 200:
            # something went horrible wrong
            return errorReqExit("GET clusters", r1.status_code)
        clusters_data = json.loads(r1.text)
        return clusters_data
    except requests.exceptions.Timeout as e:
        # Maybe set up for a retry
        print(e)
        print("ERROR: Something went wrong with get_cluster request and it reached timeout="+str(__GET_CLUSTER_TIMEOUT__)+". Maybe retry or increase timeout value.\n")


def delete_cluster_data(cluster: cFcluster):
    print("Requesting delete cluster_id={0} ...".format(cluster.get_id()))
    try:
        start = time.time()
        r1 = requests.delete(
            "http://" + __cf_manager_url__ + "/clusters/" + str(cluster.get_id()) + "?{0}".format(
                cluster.user.get_auth_string()), timeout=__DELETE_CLUSTER_TIMEOUT__)
        elapsed = time.time() - start
        print("Time for DELETE cluster: \t{0}s\n".format(elapsed))
        if r1.status_code != 204:
            # something went horrible wrong
            return errorReqExit("DELETE cluster", r1.status_code)
        return 0
    except requests.exceptions.Timeout as e:
        # Maybe set up for a retry
        print(e)
        print("ERROR: Something went wrong with delete_cluster_data request and it reached timeout="+str(__DELETE_CLUSTER_TIMEOUT__)+". Maybe retry or increase timeout value.\n")


def restart_cluster_apps(cluster: cFcluster):
    print("Requesting restart for (all) FPGA(s) of cluster_id={0} ...".format(cluster.get_id()))
    try:
        start = time.time()        
        r1 = requests.patch(
            "http://" + __cf_manager_url__ + "/clusters/" + str(cluster.get_id()) + "/restart?{0}".format(
            cluster.user.get_auth_string()))
        elapsed = time.time() - start
        print("Time for RESTART cluster: \t{0}s\n".format(elapsed))

        if r1.status_code != 200:
            # something went horrible wrong
            return errorReqExit("PATCH cluster restart", r1.status_code)
        print(r1.content.decode())

    except Exception as e:
        print("ERROR: Failed to reset the FPGA(s) role(s)")
        print(str(e))
        exit(1)

####################################################################################################
# Instances functions
####################################################################################################


class cFinstance:

    def __init__(self, user: cFuser, instance_data):
        self.instance_data = instance_data
        self.user = user
        self.id = instance_data['instance_id']

    def get_id(self):
        return self.id
    
def get_instances_data(user: cFuser, limit=100):
    print("Requesting instances data (limit="+str(limit)+")...")
    try:
        start = time.time()
        r1 = requests.get(
            "http://" + __cf_manager_url__ + "/instances" + "?{0}&limit={1}".format(
                user.get_auth_string(), limit), timeout=__GET_INSTANCE_TIMEOUT__)
        elapsed = time.time() - start
        print(r1.request.url)
        print("Time for GET instances: \t{0}s\n".format(elapsed))
        if r1.status_code != 200:
            # something went horrible wrong
            return errorReqExit("GET instances", r1.status_code)
        instances_data = json.loads(r1.text)
        return instances_data
    except requests.exceptions.Timeout as e:
        # Maybe set up for a retry
        print(e)
        print("ERROR: Something went wrong with get_instances request and it reached timeout="+str(__GET_INSTANCE_TIMEOUT__)+". Maybe retry or increase timeout value.\n")

def create_instance():
    print("TODO \n")


def get_instance_data(instance: cFinstance):
    print("Requesting instance data for instance_id={0} ...".format(instance.get_id()))
    try:
        start = time.time()
        r1 = requests.get(
            "http://" + __cf_manager_url__ + "/instances/" + str(instance.get_id()) + "?{0}".format(
                instance.user.get_auth_string()), timeout=__GET_INSTANCE_TIMEOUT__)
        elapsed = time.time() - start
        print("Time for GET instance: \t{0}s\n".format(elapsed))
        if r1.status_code != 200:
            # something went horrible wrong
            return errorReqExit("GET instance", r1.status_code)

        instance_data = json.loads(r1.text)
        # update, if necessary
        instance.instance_data = instance_data
        return instance_data
    except requests.exceptions.Timeout as e:
        # Maybe set up for a retry
        print(e)
        print("ERROR: Something went wrong with get_instance_data request and it reached timeout="+str(__GET_INSTANCE_TIMEOUT__)+". Maybe retry or increase timeout value.\n")



def reprogram_instance():
    print("TODO \n")


def api_request_instance():
    print("TODO \n")


def restart_instance_app(instance: cFinstance):
    print("Requesting restart for instance_id={0} ...".format(instance.get_id()))
    try:
        start = time.time()       
        r1 = requests.patch(
            "http://" + __cf_manager_url__ + "/instances/" + str(instance.get_id()) + "/app_restart?{0}".format(
            instance.user.get_auth_string()))
        elapsed = time.time() - start
        print("Time for RESTART instance: \t{0}s\n".format(elapsed))

        if r1.status_code != 200:
            # something went horrible wrong
            return errorReqExit("PATCH instance restart", r1.status_code)
        print(r1.content.decode())

    except Exception as e:
        print("ERROR: Failed to reset the FPGA role")
        print(str(e))
        exit(1)

def delete_instance(instance: cFinstance):
    print("deleting instance {}".format(instance.id))

    r1 = requests.delete(
        "http://" + __cf_manager_url__ + "/instances/{0}?{1}".format(instance.id, instance.user.get_auth_string()))

    if r1.status_code > 204:
        # error codes
        # 204 Instance was deleted
        # 401 Unauthenticated, bad login
        # 403 Unauthorized
        # 404 Instance does not exist
        return r1.status_code
    else:
        print("Instance {} removed".format(instance.id))

    instance_data = r1.status_code
    return instance_data


####################################################################################################
# Images functions
####################################################################################################

class cFimage:

    def __init__(self, user: cFuser, image_data):
        self.user = user
        self.image_data = image_data
        self.id = image_data['id']
        self.comment = image_data['comment']
        self.required_shell = image_data['shell_type']


def get_images(user: cFuser):
    print("TODO \n")


def get_image(image: cFimage):
    print("TODO \n")


def post_image(image: cFimage):
    print("TODO \n")


def delete_image(image: cFimage):
    print("TODO \n")


####################################################################################################
# Resources functions (admin only)
####################################################################################################

# TODO: no resource class so far, because the resource data should reside inside the CFRM, not the cFSP

def get_resource_status(resource_id, admin: cFuser):
    print("Requesting resource status...")
    r1 = requests.get(
        "http://" + __cf_manager_url__ + "/resources/" + str(
            resource_id) + "/status/" + "?{0}".format(admin.get_auth_string()))

    if r1.status_code != 200:
        # something went wrong
        return errorReqExit("GET resource status", r1.status_code)

    resource_status = json.loads(r1.text)
    return resource_status


def set_resource_status(resource_id, new_status, admin: cFuser):
    # print("set resource status")

    # possible status: "AVAILABLE", "USED", "MAINTENANCE"
    r1 = requests.put(
        "http://" + __cf_manager_url__ + "/resources/{0}/status/?{1}&new_status={2}".format(
            resource_id, admin.get_auth_string(), new_status))

    if r1.status_code != 204:
        # something went wrong
        return errorReqExit("PUT /resources/{resource_id}/status/", r1.status_code)
    else:
        print("Resource {} set to {}".format(resource_id, new_status))

    resource_data = r1.status_code
    return resource_data





# coding: utf-8

"""
    cloudFPGA Resource Manager API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.8
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Image(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'breed': 'str',
        'comment': 'str',
        'fpga_board': 'str',
        'id': 'str',
        'shell_type': 'str'
    }

    attribute_map = {
        'breed': 'breed',
        'comment': 'comment',
        'fpga_board': 'fpga_board',
        'id': 'id',
        'shell_type': 'shell_type'
    }

    def __init__(self, breed=None, comment=None, fpga_board=None, id=None, shell_type=None):  # noqa: E501
        """Image - a model defined in Swagger"""  # noqa: E501
        self._breed = None
        self._comment = None
        self._fpga_board = None
        self._id = None
        self._shell_type = None
        self.discriminator = None
        if breed is not None:
            self.breed = breed
        if comment is not None:
            self.comment = comment
        if fpga_board is not None:
            self.fpga_board = fpga_board
        if id is not None:
            self.id = id
        if shell_type is not None:
            self.shell_type = shell_type

    @property
    def breed(self):
        """Gets the breed of this Image.  # noqa: E501

        SHELL or ROLE; SHELL is only allowed by admins  # noqa: E501

        :return: The breed of this Image.  # noqa: E501
        :rtype: str
        """
        return self._breed

    @breed.setter
    def breed(self, breed):
        """Sets the breed of this Image.

        SHELL or ROLE; SHELL is only allowed by admins  # noqa: E501

        :param breed: The breed of this Image.  # noqa: E501
        :type: str
        """

        self._breed = breed

    @property
    def comment(self):
        """Gets the comment of this Image.  # noqa: E501

        Some valuable information for humans (optional)  # noqa: E501

        :return: The comment of this Image.  # noqa: E501
        :rtype: str
        """
        return self._comment

    @comment.setter
    def comment(self, comment):
        """Sets the comment of this Image.

        Some valuable information for humans (optional)  # noqa: E501

        :param comment: The comment of this Image.  # noqa: E501
        :type: str
        """

        self._comment = comment

    @property
    def fpga_board(self):
        """Gets the fpga_board of this Image.  # noqa: E501

        Type of cloudFPGA (for SHELL only)  # noqa: E501

        :return: The fpga_board of this Image.  # noqa: E501
        :rtype: str
        """
        return self._fpga_board

    @fpga_board.setter
    def fpga_board(self, fpga_board):
        """Sets the fpga_board of this Image.

        Type of cloudFPGA (for SHELL only)  # noqa: E501

        :param fpga_board: The fpga_board of this Image.  # noqa: E501
        :type: str
        """

        self._fpga_board = fpga_board

    @property
    def id(self):
        """Gets the id of this Image.  # noqa: E501

        ID of the Image  # noqa: E501

        :return: The id of this Image.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Image.

        ID of the Image  # noqa: E501

        :param id: The id of this Image.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def shell_type(self):
        """Gets the shell_type of this Image.  # noqa: E501

        Shell type as interface  # noqa: E501

        :return: The shell_type of this Image.  # noqa: E501
        :rtype: str
        """
        return self._shell_type

    @shell_type.setter
    def shell_type(self, shell_type):
        """Sets the shell_type of this Image.

        Shell type as interface  # noqa: E501

        :param shell_type: The shell_type of this Image.  # noqa: E501
        :type: str
        """

        self._shell_type = shell_type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Image, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Image):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from swagger_client.api.administration__admin_only_api import AdministrationAdminOnlyApi
from swagger_client.api.clusters_api import ClustersApi
from swagger_client.api.debug_api import DebugApi
from swagger_client.api.images_api import ImagesApi
from swagger_client.api.instances_api import InstancesApi
from swagger_client.api.mantle_architecture_api import MantleArchitectureApi
from swagger_client.api.projects_api import ProjectsApi
from swagger_client.api.resources__admin_only_api import ResourcesAdminOnlyApi



# coding: utf-8

"""
    cloudFPGA Resource Manager API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.8
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Instance(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'fpga_board': 'str',
        'image_id': 'str',
        'instance_id': 'int',
        'project_name': 'str',
        'role_ip': 'str',
        'shell_type': 'str',
        'slot_num': 'int',
        'status': 'str',
        'user_id': 'str'
    }

    attribute_map = {
        'fpga_board': 'fpga_board',
        'image_id': 'image_id',
        'instance_id': 'instance_id',
        'project_name': 'project_name',
        'role_ip': 'role_ip',
        'shell_type': 'shell_type',
        'slot_num': 'slot_num',
        'status': 'status',
        'user_id': 'user_id'
    }

    def __init__(self, fpga_board=None, image_id=None, instance_id=None, project_name=None, role_ip=None, shell_type=None, slot_num=None, status=None, user_id=None):  # noqa: E501
        """Instance - a model defined in Swagger"""  # noqa: E501
        self._fpga_board = None
        self._image_id = None
        self._instance_id = None
        self._project_name = None
        self._role_ip = None
        self._shell_type = None
        self._slot_num = None
        self._status = None
        self._user_id = None
        self.discriminator = None
        if fpga_board is not None:
            self.fpga_board = fpga_board
        if image_id is not None:
            self.image_id = image_id
        if instance_id is not None:
            self.instance_id = instance_id
        if project_name is not None:
            self.project_name = project_name
        if role_ip is not None:
            self.role_ip = role_ip
        if shell_type is not None:
            self.shell_type = shell_type
        if slot_num is not None:
            self.slot_num = slot_num
        if status is not None:
            self.status = status
        if user_id is not None:
            self.user_id = user_id

    @property
    def fpga_board(self):
        """Gets the fpga_board of this Instance.  # noqa: E501

        Type of cloudFPGA  # noqa: E501

        :return: The fpga_board of this Instance.  # noqa: E501
        :rtype: str
        """
        return self._fpga_board

    @fpga_board.setter
    def fpga_board(self, fpga_board):
        """Sets the fpga_board of this Instance.

        Type of cloudFPGA  # noqa: E501

        :param fpga_board: The fpga_board of this Instance.  # noqa: E501
        :type: str
        """

        self._fpga_board = fpga_board

    @property
    def image_id(self):
        """Gets the image_id of this Instance.  # noqa: E501

        Image ID  # noqa: E501

        :return: The image_id of this Instance.  # noqa: E501
        :rtype: str
        """
        return self._image_id

    @image_id.setter
    def image_id(self, image_id):
        """Sets the image_id of this Instance.

        Image ID  # noqa: E501

        :param image_id: The image_id of this Instance.  # noqa: E501
        :type: str
        """

        self._image_id = image_id

    @property
    def instance_id(self):
        """Gets the instance_id of this Instance.  # noqa: E501

        ID of the instance  # noqa: E501

        :return: The instance_id of this Instance.  # noqa: E501
        :rtype: int
        """
        return self._instance_id

    @instance_id.setter
    def instance_id(self, instance_id):
        """Sets the instance_id of this Instance.

        ID of the instance  # noqa: E501

        :param instance_id: The instance_id of this Instance.  # noqa: E501
        :type: int
        """

        self._instance_id = instance_id

    @property
    def project_name(self):
        """Gets the project_name of this Instance.  # noqa: E501

        Name of the OpenStack project the quota should be acounted to, if a user has multiple projects.  # noqa: E501

        :return: The project_name of this Instance.  # noqa: E501
        :rtype: str
        """
        return self._project_name

    @project_name.setter
    def project_name(self, project_name):
        """Sets the project_name of this Instance.

        Name of the OpenStack project the quota should be acounted to, if a user has multiple projects.  # noqa: E501

        :param project_name: The project_name of this Instance.  # noqa: E501
        :type: str
        """

        self._project_name = project_name

    @property
    def role_ip(self):
        """Gets the role_ip of this Instance.  # noqa: E501

        IP of sled manager  # noqa: E501

        :return: The role_ip of this Instance.  # noqa: E501
        :rtype: str
        """
        return self._role_ip

    @role_ip.setter
    def role_ip(self, role_ip):
        """Sets the role_ip of this Instance.

        IP of sled manager  # noqa: E501

        :param role_ip: The role_ip of this Instance.  # noqa: E501
        :type: str
        """

        self._role_ip = role_ip

    @property
    def shell_type(self):
        """Gets the shell_type of this Instance.  # noqa: E501

        Type of the current Shell (interface) of the FPGA  # noqa: E501

        :return: The shell_type of this Instance.  # noqa: E501
        :rtype: str
        """
        return self._shell_type

    @shell_type.setter
    def shell_type(self, shell_type):
        """Sets the shell_type of this Instance.

        Type of the current Shell (interface) of the FPGA  # noqa: E501

        :param shell_type: The shell_type of this Instance.  # noqa: E501
        :type: str
        """

        self._shell_type = shell_type

    @property
    def slot_num(self):
        """Gets the slot_num of this Instance.  # noqa: E501

        Slot number of resource  # noqa: E501

        :return: The slot_num of this Instance.  # noqa: E501
        :rtype: int
        """
        return self._slot_num

    @slot_num.setter
    def slot_num(self, slot_num):
        """Sets the slot_num of this Instance.

        Slot number of resource  # noqa: E501

        :param slot_num: The slot_num of this Instance.  # noqa: E501
        :type: int
        """

        self._slot_num = slot_num

    @property
    def status(self):
        """Gets the status of this Instance.  # noqa: E501

        Status of instance  # noqa: E501

        :return: The status of this Instance.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this Instance.

        Status of instance  # noqa: E501

        :param status: The status of this Instance.  # noqa: E501
        :type: str
        """

        self._status = status

    @property
    def user_id(self):
        """Gets the user_id of this Instance.  # noqa: E501

        User ID  # noqa: E501

        :return: The user_id of this Instance.  # noqa: E501
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this Instance.

        User ID  # noqa: E501

        :param user_id: The user_id of this Instance.  # noqa: E501
        :type: str
        """

        self._user_id = user_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Instance, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Instance):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# *****************************************************************************
# *                            cloudFPGA
# *                Copyright 2016 -- 2022 IBM Corporation
# * Licensed under the Apache License, Version 2.0 (the "License");
# * you may not use this file except in compliance with the License.
# * You may obtain a copy of the License at
# *
# *     http://www.apache.org/licenses/LICENSE-2.0
# *
# * Unless required by applicable law or agreed to in writing, software
# * distributed under the License is distributed on an "AS IS" BASIS,
# * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# * See the License for the specific language governing permissions and
# * limitations under the License.
# *----------------------------------------------------------------------------

"""
Usage: 
    cfsp instance (get | post | delete)    
Commands:
    get <id>     Get all instances of a user. Either <id> of instance or no argument for all.
    post         Request an instance.
    delete id    Delete an instance with instance_id=id. If no id is provided then all instances are deleted (after confirmation dialog with user)
"""
from __future__ import absolute_import

import sys,os
python_api_client_path = os.getcwd()+"/cFSPlib/python_api_client/"
sys.path.append(python_api_client_path)

import cfsp_globals
import swagger_client
from swagger_client.rest import ApiException
from swagger_client.api_client import ApiClient
from swagger_client.configuration import Configuration
from pprint import pprint
from tqdm import tqdm

def confirm_choice():
    confirm = input("[c]Confirm or [v]Void: ")
    if confirm != 'c' and confirm != 'v':
        print("\n Invalid Option. Please Enter a Valid Option.")
        return confirm_choice() 
    print (confirm)
    return confirm


def main(args):
    conf = Configuration()
    conf.host = cfsp_globals.__cf_manager_url__
    api_client = ApiClient(conf)    
    api_instance = swagger_client.InstancesApi(api_client=api_client)

    if ((len(args['<args>']) < 1) or (len(args['<args>']) > 2)):
        print("ERROR: invalid arguments provided in 'cfsp instance' command. Aborting...")
        exit(print(__doc__))
    
    username = cfsp_globals.__cfsp_username__
    password = cfsp_globals.__cfsp_password__
    project_name = cfsp_globals.__cfsp_project__
    
    if args['<args>'][0] == 'get':
        if (len(args['<args>']) == 2):
            try:
                api_response = api_instance.cf_manager_rest_api_get_instance(username, password, args['<args>'][1])
            except ApiException as e:
                print("Exception when calling InstancesApi->cf_manager_rest_api_get_instance: %s\n" % e)              
                exit(-1)
        elif (len(args['<args>']) == 1):
            try:
                api_response = api_instance.cf_manager_rest_api_get_instances(username, password, limit=args['--limit'])
            except ApiException as e:
                print("Exception when calling InstancesApi->cf_manager_rest_api_get_instances: %s\n" % e)              
                exit(-1)            
        else:
            exit(print("ERROR: invalid arguments provided in cfsp instance get. Aborting..."))
        pprint(api_response)
    elif args['<args>'][0] == 'post':
        # create an instance of the API class
        if len(args['--image_id']) != 1:
            exit(print("ERROR: instance post supports only one image (--image_id). Aborting..."))
        image_id = args['--image_id'][0]
        try:
            # Request an instance
            api_response = api_instance.cf_manager_rest_api_post_instances(image_id, username, password, project_name=project_name, dont_verify_memory=args['--dont_verify_memory'])
            # post_instance dows not return the role_ip, thus we call get_instance afterwards.
            instance_id = api_response.instance_id
            try:
                api_response = api_instance.cf_manager_rest_api_get_instance(username, password, instance_id)
            except ApiException as e:
                print("Exception when calling InstancesApi->cf_manager_rest_api_get_instance: %s\n" % e)              
                exit(-1)
            pprint(api_response)
        except ApiException as e:
            print("Exception when calling InstancesApi->cf_manager_rest_api_post_instances: %s\n" % e)
            exit(-1)
    elif args['<args>'][0] == 'delete':
        if (len(args['<args>']) == 1):
            print("INFO: Really deleting all instances ?")
            if confirm_choice() == 'c':
                print("INFO: Confirmed deleting all instances")
                try:
                    api_response_get_in_delete = api_instance.cf_manager_rest_api_get_instances(username, password, limit=args['--limit'])
                    if(len(api_response_get_in_delete) > 0):
                        for this_instance in tqdm(api_response_get_in_delete):                                  
                            # Delete an instance
                            print("INFO: Deleting instance " + str(this_instance.instance_id) + " ... ")
                            try:
                                api_instance.cf_manager_rest_api_delete_instance(username, password, this_instance.instance_id)
                            except ApiException as e_in_delete:
                                print("Exception when calling InstancesApi->cf_manager_rest_api_delete_instance: %s\n" % e_in_delete)        
                    else:
                        print("INFO: No instances to delete.")
                except ApiException as e_in_get:
                    print("Exception when calling InstancesApi->cf_manager_rest_api_get_instances: %s\n" % e_in_get)              
                    exit(-1)
            else:
                print ("INFO: Canceling deleting all instances")
        elif (len(args['<args>']) == 2):
            instance_id = args['<args>'][1]
            # Delete an instance
            print("INFO: Deleting instance " + str(instance_id) + " ... ")
            try:
                api_instance.cf_manager_rest_api_delete_instance(username, password, instance_id)
            except ApiException as e:
                print("Exception when calling InstancesApi->cf_manager_rest_api_delete_instance: %s\n" % e)
                exit(-1)
        else:
            exit(print("ERROR: invalid arguments provided in cfsp instance delete. Aborting..."))
    else:
        exit(print("ERROR: invalid command provided in cfsp instance. Type 'cfsp help instance' to get a list of supported commands. Aborting..."))
if __name__ == '__main__':
    main(args)



# *****************************************************************************
# *                            cloudFPGA
# *                Copyright 2016 -- 2022 IBM Corporation
# * Licensed under the Apache License, Version 2.0 (the "License");
# * you may not use this file except in compliance with the License.
# * You may obtain a copy of the License at
# *
# *     http://www.apache.org/licenses/LICENSE-2.0
# *
# * Unless required by applicable law or agreed to in writing, software
# * distributed under the License is distributed on an "AS IS" BASIS,
# * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# * See the License for the specific language governing permissions and
# * limitations under the License.
# *----------------------------------------------------------------------------

"""
Usage: 
    cfsp cluster (get | post | delete)    
Commands:
    get <id>     Get all clusters of a user. Either <id> of cluster or no argument for all.
    post         Request a cluster.
    delete id    Delete a cluster with cluster_id=id. If no id is provided then all clusters are deleted (after confirmation dialog with user)
"""
from __future__ import absolute_import

import sys,os
#python_api_client_path = os.getcwd()+"/cFSPlib/python_api_client/"
python_api_client_path = os.path.dirname(os.path.abspath(__file__))+"/python_api_client/"

sys.path.append(python_api_client_path)

import cfsp_globals
import swagger_client
from swagger_client.api.clusters_api import ClustersApi  # noqa: E501
from swagger_client.rest import ApiException
from swagger_client.api_client import ApiClient
from swagger_client.configuration import Configuration
from pprint import pprint
from tqdm import tqdm

def confirm_choice():
    confirm = input("[c]Confirm or [v]Void: ")
    if confirm != 'c' and confirm != 'v':
        print("\n Invalid Option. Please Enter a Valid Option.")
        return confirm_choice() 
    print (confirm)
    return confirm


def main(args):
    conf = Configuration()
    conf.host = cfsp_globals.__cf_manager_url__
    api_client = ApiClient(conf)    
    api_instance = ClustersApi(api_client=api_client) 

    if ((len(args['<args>']) < 1) or (len(args['<args>']) > 2)):
        print("ERROR: invalid arguments provided in 'cfsp cluster' command. Aborting...")
        exit(print(__doc__))
    
    username = cfsp_globals.__cfsp_username__
    password = cfsp_globals.__cfsp_password__
    project_name = cfsp_globals.__cfsp_project__

    if args['<args>'][0] == 'get':
        if (len(args['<args>']) == 2):
            try:
                api_response = api_instance.cf_manager_rest_api_get_cluster_single(username, password, args['<args>'][1])
            except ApiException as e:
                print("Exception when calling ClustersApi->cf_manager_rest_api_get_cluster_single: %s\n" % e)              
                exit(-1)
        elif (len(args['<args>']) == 1):
            try:
                api_response = api_instance.cf_manager_rest_api_get_clusters(username, password, limit=args['--limit'])
            except ApiException as e:
                print("Exception when calling ClustersApi->cf_manager_rest_api_get_clusters: %s\n" % e)              
                exit(-1)            
        else:
            exit(print("ERROR: invalid arguments provided in cfsp cluster get. Aborting..."))
        pprint(api_response)
    elif args['<args>'][0] == 'post':
        # create an instance of the API class
        body = []        
        cpu_num = len(args['--node_ip'])
        fpga_num = len(args['--image_id'])
        node_id_num = len(args['--node_id'])
        
        if node_id_num == 0:
            print("WARNING: No --node_id argument was provided. Incremental ids will be used for fpga(s) and cpu(s) in this cluster.")
            for i in range(fpga_num):
                args['--node_id'].append(i)
            for j in range(cpu_num):
                args['--node_id'].append(fpga_num+j)
        else:
            if (node_id_num != cpu_num + fpga_num):
                exit(print("ERROR: The provided argument(s) of --node_id ("+str(node_id_num)+") is not the same with the sum of --node_ip ("+str(cpu_num)+") and --image_id ("+str(fpga_num)+") ones. Please note that for each of --node_ip and --image_id, a --node_id argument is required. Aborting..."))
        
        # Convert the node_id list to ints
        args['--node_id'] = list(map(int, args['--node_id']))
   
        print("INFO: Please review the assignment of image_id(s), node_ip(s) and node_id(s)")
        print("[image_id, node_id]")
        for i in range(fpga_num):
            fpga_body = {    "image_id": args['--image_id'][i],  "node_id": args['--node_id'][i] }
            print("["+args['--image_id'][i] + ", " + str(args['--node_id'][i]) + "]")
            body.append(fpga_body)
        print("[node_ip, node_id]")            
        for j in range(cpu_num):
            cpu_body = {    "image_id": cfsp_globals.__NON_FPGA_IDENTIFIER__,    "node_ip":  args['--node_ip'][j],    "node_id": args['--node_id'][fpga_num+j] }
            print("["+args['--node_ip'][j] + ", " + str(args['--node_id'][fpga_num+j]) + "]")
            body.append(cpu_body)
        
        try:
            # Request a cluster
            api_response = api_instance.cf_manager_rest_api_post_clusters(body, username, password, project_name=project_name, dont_verify_memory=args['--dont_verify_memory'])
            pprint(api_response)
        except ApiException as e:
            print("Exception when calling ClustersApi->cf_manager_rest_api_post_clusters: %s\n" % e)
            exit(-1)
    elif args['<args>'][0] == 'delete':
        if (len(args['<args>']) == 1):
            print("INFO: Really deleting all clusters ?")
            if confirm_choice() == 'c':
                print("INFO: Confirmed deleting all clusters")
                try:
                    api_response_get_in_delete = api_instance.cf_manager_rest_api_get_clusters(username, password)
                    if(len(api_response_get_in_delete) > 0):
                        for this_cluster in tqdm(api_response_get_in_delete):                                  
                            # Delete a cluster
                            print("INFO: Deleting cluster " + str(this_cluster.cluster_id) + " ... ")
                            try:
                                api_instance.cf_manager_rest_api_delete_cluster(username, password, this_cluster.cluster_id)
                            except ApiException as e_in_delete:
                                print("Exception when calling ClustersApi->cf_manager_rest_api_delete_cluster: %s\n" % e_in_delete)        
                    else:
                        print("INFO: No clusters to delete.")
                except ApiException as e_in_get:
                    print("Exception when calling ClustersApi->cf_manager_rest_api_get_clusters: %s\n" % e_in_get)              
                    exit(-1)
            else:
                print ("INFO: Canceling deleting all clsuters")
        elif (len(args['<args>']) == 2):
            cluster_id = args['<args>'][1]
            # Delete a cluster
            print("INFO: Deleting cluster " + str(cluster_id) + " ... ")
            try:
                api_instance.cf_manager_rest_api_delete_cluster(username, password, cluster_id)
            except ApiException as e:
                print("Exception when calling ClustersApi->cf_manager_rest_api_delete_cluster: %s\n" % e)
                exit(-1)
        else:
            exit(print("ERROR: invalid arguments provided in cfsp cluster delete. Aborting..."))
    else:
        exit(print("ERROR: invalid command provided in cfsp cluster. Type 'cfsp help cluster' to get a list of supported commands. Aborting..."))            

if __name__ == '__main__':
    main(args)



# coding: utf-8

"""
    cloudFPGA Resource Manager API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.8
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class InlineResponse2001(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'image_id': 'str',
        'image_ip': 'str',
        'instance_id': 'int',
        'slot_num': 'int',
        'staus': 'str'
    }

    attribute_map = {
        'image_id': 'image_id',
        'image_ip': 'image_ip',
        'instance_id': 'instance_id',
        'slot_num': 'slot_num',
        'staus': 'staus'
    }

    def __init__(self, image_id=None, image_ip=None, instance_id=None, slot_num=None, staus=None):  # noqa: E501
        """InlineResponse2001 - a model defined in Swagger"""  # noqa: E501
        self._image_id = None
        self._image_ip = None
        self._instance_id = None
        self._slot_num = None
        self._staus = None
        self.discriminator = None
        if image_id is not None:
            self.image_id = image_id
        if image_ip is not None:
            self.image_ip = image_ip
        if instance_id is not None:
            self.instance_id = instance_id
        if slot_num is not None:
            self.slot_num = slot_num
        if staus is not None:
            self.staus = staus

    @property
    def image_id(self):
        """Gets the image_id of this InlineResponse2001.  # noqa: E501

        Image unique identifier  # noqa: E501

        :return: The image_id of this InlineResponse2001.  # noqa: E501
        :rtype: str
        """
        return self._image_id

    @image_id.setter
    def image_id(self, image_id):
        """Sets the image_id of this InlineResponse2001.

        Image unique identifier  # noqa: E501

        :param image_id: The image_id of this InlineResponse2001.  # noqa: E501
        :type: str
        """

        self._image_id = image_id

    @property
    def image_ip(self):
        """Gets the image_ip of this InlineResponse2001.  # noqa: E501

        Ip Address of the SHELL  # noqa: E501

        :return: The image_ip of this InlineResponse2001.  # noqa: E501
        :rtype: str
        """
        return self._image_ip

    @image_ip.setter
    def image_ip(self, image_ip):
        """Sets the image_ip of this InlineResponse2001.

        Ip Address of the SHELL  # noqa: E501

        :param image_ip: The image_ip of this InlineResponse2001.  # noqa: E501
        :type: str
        """

        self._image_ip = image_ip

    @property
    def instance_id(self):
        """Gets the instance_id of this InlineResponse2001.  # noqa: E501

        ROLE instance unique identifier  # noqa: E501

        :return: The instance_id of this InlineResponse2001.  # noqa: E501
        :rtype: int
        """
        return self._instance_id

    @instance_id.setter
    def instance_id(self, instance_id):
        """Sets the instance_id of this InlineResponse2001.

        ROLE instance unique identifier  # noqa: E501

        :param instance_id: The instance_id of this InlineResponse2001.  # noqa: E501
        :type: int
        """

        self._instance_id = instance_id

    @property
    def slot_num(self):
        """Gets the slot_num of this InlineResponse2001.  # noqa: E501

        Number of the deployed slot  # noqa: E501

        :return: The slot_num of this InlineResponse2001.  # noqa: E501
        :rtype: int
        """
        return self._slot_num

    @slot_num.setter
    def slot_num(self, slot_num):
        """Sets the slot_num of this InlineResponse2001.

        Number of the deployed slot  # noqa: E501

        :param slot_num: The slot_num of this InlineResponse2001.  # noqa: E501
        :type: int
        """

        self._slot_num = slot_num

    @property
    def staus(self):
        """Gets the staus of this InlineResponse2001.  # noqa: E501

        Reports that the instance was sucessfull deployed.  # noqa: E501

        :return: The staus of this InlineResponse2001.  # noqa: E501
        :rtype: str
        """
        return self._staus

    @staus.setter
    def staus(self, staus):
        """Sets the staus of this InlineResponse2001.

        Reports that the instance was sucessfull deployed.  # noqa: E501

        :param staus: The staus of this InlineResponse2001.  # noqa: E501
        :type: str
        """

        self._staus = staus

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(InlineResponse2001, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, InlineResponse2001):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    cloudFPGA Resource Manager API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.8
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

import swagger_client
from swagger_client.api.resources__admin_only_api import ResourcesAdminOnlyApi  # noqa: E501
from swagger_client.rest import ApiException


class TestResourcesAdminOnlyApi(unittest.TestCase):
    """ResourcesAdminOnlyApi unit test stubs"""

    def setUp(self):
        self.api = ResourcesAdminOnlyApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_cf_manager_rest_api_delete_resource(self):
        """Test case for cf_manager_rest_api_delete_resource

        Remove a resource  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_get_available_resources(self):
        """Test case for cf_manager_rest_api_get_available_resources

        Get all cloudFPGA resources in state `{status}`  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_get_resources(self):
        """Test case for cf_manager_rest_api_get_resources

        Get all cloudFPGA resources  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_get_resources_of_sled_status(self):
        """Test case for cf_manager_rest_api_get_resources_of_sled_status

        Get status of **all resources** from a specific sled  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_get_resources_of_sled_status_in_state(self):
        """Test case for cf_manager_rest_api_get_resources_of_sled_status_in_state

        Get status of **all resources** from a specific sled in a specific `{status}`  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_get_single_resource(self):
        """Test case for cf_manager_rest_api_get_single_resource

        Get details of one resource  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_get_single_resource_status(self):
        """Test case for cf_manager_rest_api_get_single_resource_status

        Get status of one resource  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_post_resources(self):
        """Test case for cf_manager_rest_api_post_resources

        Create a cloudFPGA resource  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_put_resource(self):
        """Test case for cf_manager_rest_api_put_resource

        Update a resource  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_put_resource_status(self):
        """Test case for cf_manager_rest_api_put_resource_status

        Update status of a resource  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_put_resources_of_sled_status(self):
        """Test case for cf_manager_rest_api_put_resources_of_sled_status

        Update status of **all resources** from a specific sled  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_put_resources_of_sled_status_in_state(self):
        """Test case for cf_manager_rest_api_put_resources_of_sled_status_in_state

        Update status of **all resources** from a specific sled in a specific `{status}`  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    cloudFPGA Resource Manager API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.8
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Quota(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'allowed_resources': 'list[int]',
        'current_acquired_instances': 'int',
        'max_acquire_instances': 'int'
    }

    attribute_map = {
        'allowed_resources': 'allowed_resources',
        'current_acquired_instances': 'current_acquired_instances',
        'max_acquire_instances': 'max_acquire_instances'
    }

    def __init__(self, allowed_resources=None, current_acquired_instances=None, max_acquire_instances=None):  # noqa: E501
        """Quota - a model defined in Swagger"""  # noqa: E501
        self._allowed_resources = None
        self._current_acquired_instances = None
        self._max_acquire_instances = None
        self.discriminator = None
        if allowed_resources is not None:
            self.allowed_resources = allowed_resources
        if current_acquired_instances is not None:
            self.current_acquired_instances = current_acquired_instances
        if max_acquire_instances is not None:
            self.max_acquire_instances = max_acquire_instances

    @property
    def allowed_resources(self):
        """Gets the allowed_resources of this Quota.  # noqa: E501

        List of allowed resources for this project (admin only).  # noqa: E501

        :return: The allowed_resources of this Quota.  # noqa: E501
        :rtype: list[int]
        """
        return self._allowed_resources

    @allowed_resources.setter
    def allowed_resources(self, allowed_resources):
        """Sets the allowed_resources of this Quota.

        List of allowed resources for this project (admin only).  # noqa: E501

        :param allowed_resources: The allowed_resources of this Quota.  # noqa: E501
        :type: list[int]
        """

        self._allowed_resources = allowed_resources

    @property
    def current_acquired_instances(self):
        """Gets the current_acquired_instances of this Quota.  # noqa: E501

        Number of currently acquired Instances  # noqa: E501

        :return: The current_acquired_instances of this Quota.  # noqa: E501
        :rtype: int
        """
        return self._current_acquired_instances

    @current_acquired_instances.setter
    def current_acquired_instances(self, current_acquired_instances):
        """Sets the current_acquired_instances of this Quota.

        Number of currently acquired Instances  # noqa: E501

        :param current_acquired_instances: The current_acquired_instances of this Quota.  # noqa: E501
        :type: int
        """

        self._current_acquired_instances = current_acquired_instances

    @property
    def max_acquire_instances(self):
        """Gets the max_acquire_instances of this Quota.  # noqa: E501

        Maximum allowed number of Instaces to be acquired by project  # noqa: E501

        :return: The max_acquire_instances of this Quota.  # noqa: E501
        :rtype: int
        """
        return self._max_acquire_instances

    @max_acquire_instances.setter
    def max_acquire_instances(self, max_acquire_instances):
        """Sets the max_acquire_instances of this Quota.

        Maximum allowed number of Instaces to be acquired by project  # noqa: E501

        :param max_acquire_instances: The max_acquire_instances of this Quota.  # noqa: E501
        :type: int
        """

        self._max_acquire_instances = max_acquire_instances

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Quota, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Quota):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    cloudFPGA Resource Manager API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.8
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

import swagger_client
from swagger_client.api.instances_api import InstancesApi  # noqa: E501
from swagger_client.rest import ApiException


class TestInstancesApi(unittest.TestCase):
    """InstancesApi unit test stubs"""

    def setUp(self):
        self.api = InstancesApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_cf_manager_rest_api_app_restart_instance(self):
        """Test case for cf_manager_rest_api_app_restart_instance

        Triggers app restart of this instance  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_delete_instance(self):
        """Test case for cf_manager_rest_api_delete_instance

        Remove an instance  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_get_instance(self):
        """Test case for cf_manager_rest_api_get_instance

        Get a single instance  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_get_instances(self):
        """Test case for cf_manager_rest_api_get_instances

        Get all instances of a user  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_instance_api_gateway(self):
        """Test case for cf_manager_rest_api_instance_api_gateway

        Forwards a custom HTTP API request  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_instance_change_runlevel(self):
        """Test case for cf_manager_rest_api_instance_change_runlevel

        Changes the `runlevel` of the instance (stack developers only)  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_instance_get_runlevel(self):
        """Test case for cf_manager_rest_api_instance_get_runlevel

        Returns the current `runlevel` of the instance (stack developers only)  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_post_instances(self):
        """Test case for cf_manager_rest_api_post_instances

        Create an instance  # noqa: E501
        """
        pass

    def test_cf_manager_rest_api_update_instance(self):
        """Test case for cf_manager_rest_api_update_instance

        Reprogramm an instance  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    cloudFPGA Resource Manager API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.8
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class AdministrationMantleLogicBody(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'image_details': 'str',
        'bin_file': 'str',
        'sig_file': 'str',
        'pr_verify_rpt': 'str'
    }

    attribute_map = {
        'image_details': 'image_details',
        'bin_file': 'bin_file',
        'sig_file': 'sig_file',
        'pr_verify_rpt': 'pr_verify_rpt'
    }

    def __init__(self, image_details=None, bin_file=None, sig_file=None, pr_verify_rpt=None):  # noqa: E501
        """AdministrationMantleLogicBody - a model defined in Swagger"""  # noqa: E501
        self._image_details = None
        self._bin_file = None
        self._sig_file = None
        self._pr_verify_rpt = None
        self.discriminator = None
        self.image_details = image_details
        self.bin_file = bin_file
        self.sig_file = sig_file
        self.pr_verify_rpt = pr_verify_rpt

    @property
    def image_details(self):
        """Gets the image_details of this AdministrationMantleLogicBody.  # noqa: E501

        Must be a valid `image_detail` dict-representation. Example: ```json  {    \"cl_id\": \"4711\",    \"version\":    \"0.1.2\",    \"name\":      \"MPIenc v2\",    \"comment\" : \"Some valuable               information for humans (optional)\"  }  ```   # noqa: E501

        :return: The image_details of this AdministrationMantleLogicBody.  # noqa: E501
        :rtype: str
        """
        return self._image_details

    @image_details.setter
    def image_details(self, image_details):
        """Sets the image_details of this AdministrationMantleLogicBody.

        Must be a valid `image_detail` dict-representation. Example: ```json  {    \"cl_id\": \"4711\",    \"version\":    \"0.1.2\",    \"name\":      \"MPIenc v2\",    \"comment\" : \"Some valuable               information for humans (optional)\"  }  ```   # noqa: E501

        :param image_details: The image_details of this AdministrationMantleLogicBody.  # noqa: E501
        :type: str
        """
        if image_details is None:
            raise ValueError("Invalid value for `image_details`, must not be `None`")  # noqa: E501

        self._image_details = image_details

    @property
    def bin_file(self):
        """Gets the bin_file of this AdministrationMantleLogicBody.  # noqa: E501

        The corresponding dynamic binfile for PR  # noqa: E501

        :return: The bin_file of this AdministrationMantleLogicBody.  # noqa: E501
        :rtype: str
        """
        return self._bin_file

    @bin_file.setter
    def bin_file(self, bin_file):
        """Sets the bin_file of this AdministrationMantleLogicBody.

        The corresponding dynamic binfile for PR  # noqa: E501

        :param bin_file: The bin_file of this AdministrationMantleLogicBody.  # noqa: E501
        :type: str
        """
        if bin_file is None:
            raise ValueError("Invalid value for `bin_file`, must not be `None`")  # noqa: E501

        self._bin_file = bin_file

    @property
    def sig_file(self):
        """Gets the sig_file of this AdministrationMantleLogicBody.  # noqa: E501

        The corresponding .sig file of the binfile  # noqa: E501

        :return: The sig_file of this AdministrationMantleLogicBody.  # noqa: E501
        :rtype: str
        """
        return self._sig_file

    @sig_file.setter
    def sig_file(self, sig_file):
        """Sets the sig_file of this AdministrationMantleLogicBody.

        The corresponding .sig file of the binfile  # noqa: E501

        :param sig_file: The sig_file of this AdministrationMantleLogicBody.  # noqa: E501
        :type: str
        """
        if sig_file is None:
            raise ValueError("Invalid value for `sig_file`, must not be `None`")  # noqa: E501

        self._sig_file = sig_file

    @property
    def pr_verify_rpt(self):
        """Gets the pr_verify_rpt of this AdministrationMantleLogicBody.  # noqa: E501

        Result of the `pr_verify` command  # noqa: E501

        :return: The pr_verify_rpt of this AdministrationMantleLogicBody.  # noqa: E501
        :rtype: str
        """
        return self._pr_verify_rpt

    @pr_verify_rpt.setter
    def pr_verify_rpt(self, pr_verify_rpt):
        """Sets the pr_verify_rpt of this AdministrationMantleLogicBody.

        Result of the `pr_verify` command  # noqa: E501

        :param pr_verify_rpt: The pr_verify_rpt of this AdministrationMantleLogicBody.  # noqa: E501
        :type: str
        """
        if pr_verify_rpt is None:
            raise ValueError("Invalid value for `pr_verify_rpt`, must not be `None`")  # noqa: E501

        self._pr_verify_rpt = pr_verify_rpt

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(AdministrationMantleLogicBody, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AdministrationMantleLogicBody):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    cloudFPGA Resource Manager API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 0.8
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from swagger_client.api_client import ApiClient


class AdministrationAdminOnlyApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def cf_manager_rest_api_admin_test_instance(self, body, username, password, instance_id, **kwargs):  # noqa: E501
        """Test the instance **if not used**  # noqa: E501

        This call helps admins to test a specific instance, **if this instance is not used by another user**. This is **intended only for single instances**, not to update one instance out of a cluster.  **If this call fails with `507`** and the FPGA worked without any problems before, then the submitted image probably doesn't work (i.e. `507` doesn't imply a failure of the FPGA board). This will also assign a new ip address to this instance.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_admin_test_instance(body, username, password, instance_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param TestInstanceInstanceIdBody body: (required)
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param str instance_id: ROLE instance unique identifier (required)
        :param int dont_verify_memory: If 1, don't verify the DDR4 memory during setup
        :return: InlineResponse2001
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_admin_test_instance_with_http_info(body, username, password, instance_id, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_admin_test_instance_with_http_info(body, username, password, instance_id, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_admin_test_instance_with_http_info(self, body, username, password, instance_id, **kwargs):  # noqa: E501
        """Test the instance **if not used**  # noqa: E501

        This call helps admins to test a specific instance, **if this instance is not used by another user**. This is **intended only for single instances**, not to update one instance out of a cluster.  **If this call fails with `507`** and the FPGA worked without any problems before, then the submitted image probably doesn't work (i.e. `507` doesn't imply a failure of the FPGA board). This will also assign a new ip address to this instance.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_admin_test_instance_with_http_info(body, username, password, instance_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param TestInstanceInstanceIdBody body: (required)
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param str instance_id: ROLE instance unique identifier (required)
        :param int dont_verify_memory: If 1, don't verify the DDR4 memory during setup
        :return: InlineResponse2001
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'username', 'password', 'instance_id', 'dont_verify_memory']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_admin_test_instance" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or
                params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `cf_manager_rest_api_admin_test_instance`")  # noqa: E501
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_admin_test_instance`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_admin_test_instance`")  # noqa: E501
        # verify the required parameter 'instance_id' is set
        if ('instance_id' not in params or
                params['instance_id'] is None):
            raise ValueError("Missing the required parameter `instance_id` when calling `cf_manager_rest_api_admin_test_instance`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'instance_id' in params:
            path_params['instance_id'] = params['instance_id']  # noqa: E501

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501
        if 'dont_verify_memory' in params:
            query_params.append(('dont_verify_memory', params['dont_verify_memory']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/test_instance/{instance_id}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='InlineResponse2001',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cf_manager_rest_api_admin_update_flash(self, username, password, resource_id, pl_id, **kwargs):  # noqa: E501
        """Update the flash of a resource **if not used**  # noqa: E501

        This call allows admins to update the flash file of a resource, **if this instance is not used by another user**. If successful, the resource's shell_type is updated and the resource is marked as AVAILABLE.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_admin_update_flash(username, password, resource_id, pl_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param str resource_id: cloudFPGA resource unique identifier (required)
        :param str pl_id: Platform logic unique identifier (required)
        :return: Resource
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_admin_update_flash_with_http_info(username, password, resource_id, pl_id, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_admin_update_flash_with_http_info(username, password, resource_id, pl_id, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_admin_update_flash_with_http_info(self, username, password, resource_id, pl_id, **kwargs):  # noqa: E501
        """Update the flash of a resource **if not used**  # noqa: E501

        This call allows admins to update the flash file of a resource, **if this instance is not used by another user**. If successful, the resource's shell_type is updated and the resource is marked as AVAILABLE.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_admin_update_flash_with_http_info(username, password, resource_id, pl_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param str resource_id: cloudFPGA resource unique identifier (required)
        :param str pl_id: Platform logic unique identifier (required)
        :return: Resource
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'password', 'resource_id', 'pl_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_admin_update_flash" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_admin_update_flash`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_admin_update_flash`")  # noqa: E501
        # verify the required parameter 'resource_id' is set
        if ('resource_id' not in params or
                params['resource_id'] is None):
            raise ValueError("Missing the required parameter `resource_id` when calling `cf_manager_rest_api_admin_update_flash`")  # noqa: E501
        # verify the required parameter 'pl_id' is set
        if ('pl_id' not in params or
                params['pl_id'] is None):
            raise ValueError("Missing the required parameter `pl_id` when calling `cf_manager_rest_api_admin_update_flash`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'resource_id' in params:
            path_params['resource_id'] = params['resource_id']  # noqa: E501

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501
        if 'pl_id' in params:
            query_params.append(('pl_id', params['pl_id']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/update_flash/{resource_id}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Resource',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cf_manager_rest_api_delete_ippool(self, username, password, subnet_string, **kwargs):  # noqa: E501
        """Remove a /24 subnet  # noqa: E501

        Marks the submitted `/24` subnet as `DELETED`. *Note: this will not delete all ip addresses immediately*, `USED` ip addresses remains used until the corresponding `instance` is deleted. For now *only /24 subnets* are supported, so it is enough to submit the subnet base, *e.g. `10.12.200.0`*!   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_delete_ippool(username, password, subnet_string, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param str subnet_string: A /24 subnet (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_delete_ippool_with_http_info(username, password, subnet_string, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_delete_ippool_with_http_info(username, password, subnet_string, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_delete_ippool_with_http_info(self, username, password, subnet_string, **kwargs):  # noqa: E501
        """Remove a /24 subnet  # noqa: E501

        Marks the submitted `/24` subnet as `DELETED`. *Note: this will not delete all ip addresses immediately*, `USED` ip addresses remains used until the corresponding `instance` is deleted. For now *only /24 subnets* are supported, so it is enough to submit the subnet base, *e.g. `10.12.200.0`*!   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_delete_ippool_with_http_info(username, password, subnet_string, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param str subnet_string: A /24 subnet (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'password', 'subnet_string']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_delete_ippool" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_delete_ippool`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_delete_ippool`")  # noqa: E501
        # verify the required parameter 'subnet_string' is set
        if ('subnet_string' not in params or
                params['subnet_string'] is None):
            raise ValueError("Missing the required parameter `subnet_string` when calling `cf_manager_rest_api_delete_ippool`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501
        if 'subnet_string' in params:
            query_params.append(('subnet_string', params['subnet_string']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/ippool/', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cf_manager_rest_api_get_all_composable_logic(self, username, password, **kwargs):  # noqa: E501
        """Get list of all composable logic  # noqa: E501

        This API call also checks if DEPRECATED or DISCHARGED PLs are still programmed to some resources.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_get_all_composable_logic(username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_get_all_composable_logic_with_http_info(username, password, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_get_all_composable_logic_with_http_info(username, password, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_get_all_composable_logic_with_http_info(self, username, password, **kwargs):  # noqa: E501
        """Get list of all composable logic  # noqa: E501

        This API call also checks if DEPRECATED or DISCHARGED PLs are still programmed to some resources.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_get_all_composable_logic_with_http_info(username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'password']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_get_all_composable_logic" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_get_all_composable_logic`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_get_all_composable_logic`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/composablelogic/all/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cf_manager_rest_api_get_buildscripts_all(self, username, password, **kwargs):  # noqa: E501
        """Get all registered build script versions  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_get_buildscripts_all(username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :return: list[BuildScript]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_get_buildscripts_all_with_http_info(username, password, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_get_buildscripts_all_with_http_info(username, password, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_get_buildscripts_all_with_http_info(self, username, password, **kwargs):  # noqa: E501
        """Get all registered build script versions  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_get_buildscripts_all_with_http_info(username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :return: list[BuildScript]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'password']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_get_buildscripts_all" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_get_buildscripts_all`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_get_buildscripts_all`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/buildscripts', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[BuildScript]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cf_manager_rest_api_get_ippool(self, username, password, **kwargs):  # noqa: E501
        """Get all ip addresses in the current pool  # noqa: E501

        This call returns a list of *all* ip addresses that are currently known, regardless their state. That includes also ip addresses that are marked as `DELETED` but are still in use.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_get_ippool(username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :return: list[InlineResponse200]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_get_ippool_with_http_info(username, password, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_get_ippool_with_http_info(username, password, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_get_ippool_with_http_info(self, username, password, **kwargs):  # noqa: E501
        """Get all ip addresses in the current pool  # noqa: E501

        This call returns a list of *all* ip addresses that are currently known, regardless their state. That includes also ip addresses that are marked as `DELETED` but are still in use.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_get_ippool_with_http_info(username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :return: list[InlineResponse200]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'password']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_get_ippool" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_get_ippool`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_get_ippool`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/ippool/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[InlineResponse200]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cf_manager_rest_api_get_mantle_logic_all(self, username, password, **kwargs):  # noqa: E501
        """Get all mantle logics  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_get_mantle_logic_all(username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_get_mantle_logic_all_with_http_info(username, password, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_get_mantle_logic_all_with_http_info(username, password, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_get_mantle_logic_all_with_http_info(self, username, password, **kwargs):  # noqa: E501
        """Get all mantle logics  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_get_mantle_logic_all_with_http_info(username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'password']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_get_mantle_logic_all" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_get_mantle_logic_all`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_get_mantle_logic_all`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/mantle_logic', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cf_manager_rest_api_get_platform_logic_all(self, username, password, **kwargs):  # noqa: E501
        """Get all platform logics  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_get_platform_logic_all(username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_get_platform_logic_all_with_http_info(username, password, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_get_platform_logic_all_with_http_info(username, password, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_get_platform_logic_all_with_http_info(self, username, password, **kwargs):  # noqa: E501
        """Get all platform logics  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_get_platform_logic_all_with_http_info(username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'password']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_get_platform_logic_all" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_get_platform_logic_all`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_get_platform_logic_all`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/platform_logic', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cf_manager_rest_api_get_single_buildscript(self, username, password, bs_id, **kwargs):  # noqa: E501
        """Get details of one registered build script  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_get_single_buildscript(username, password, bs_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param int bs_id: ID of a registered build script (required)
        :return: BuildScript
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_get_single_buildscript_with_http_info(username, password, bs_id, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_get_single_buildscript_with_http_info(username, password, bs_id, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_get_single_buildscript_with_http_info(self, username, password, bs_id, **kwargs):  # noqa: E501
        """Get details of one registered build script  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_get_single_buildscript_with_http_info(username, password, bs_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param int bs_id: ID of a registered build script (required)
        :return: BuildScript
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'password', 'bs_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_get_single_buildscript" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_get_single_buildscript`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_get_single_buildscript`")  # noqa: E501
        # verify the required parameter 'bs_id' is set
        if ('bs_id' not in params or
                params['bs_id'] is None):
            raise ValueError("Missing the required parameter `bs_id` when calling `cf_manager_rest_api_get_single_buildscript`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'bs_id' in params:
            path_params['bs_id'] = params['bs_id']  # noqa: E501

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/buildscripts/{bs_id}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='BuildScript',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cf_manager_rest_api_get_single_composable_logic(self, username, password, cl_id, **kwargs):  # noqa: E501
        """Get details of one composable logic  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_get_single_composable_logic(username, password, cl_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param int cl_id: ID of a composable logic (Static Shell or Mantles) (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_get_single_composable_logic_with_http_info(username, password, cl_id, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_get_single_composable_logic_with_http_info(username, password, cl_id, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_get_single_composable_logic_with_http_info(self, username, password, cl_id, **kwargs):  # noqa: E501
        """Get details of one composable logic  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_get_single_composable_logic_with_http_info(username, password, cl_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param int cl_id: ID of a composable logic (Static Shell or Mantles) (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'password', 'cl_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_get_single_composable_logic" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_get_single_composable_logic`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_get_single_composable_logic`")  # noqa: E501
        # verify the required parameter 'cl_id' is set
        if ('cl_id' not in params or
                params['cl_id'] is None):
            raise ValueError("Missing the required parameter `cl_id` when calling `cf_manager_rest_api_get_single_composable_logic`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'cl_id' in params:
            path_params['cl_id'] = params['cl_id']  # noqa: E501

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/composablelogic/{cl_id}/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cf_manager_rest_api_post_buildscript(self, username, password, **kwargs):  # noqa: E501
        """Register a new build script version  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_post_buildscript(username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param BuildScript body:
        :return: BuildScript
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_post_buildscript_with_http_info(username, password, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_post_buildscript_with_http_info(username, password, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_post_buildscript_with_http_info(self, username, password, **kwargs):  # noqa: E501
        """Register a new build script version  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_post_buildscript_with_http_info(username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param BuildScript body:
        :return: BuildScript
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'password', 'body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_post_buildscript" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_post_buildscript`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_post_buildscript`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/buildscripts', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='BuildScript',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cf_manager_rest_api_post_ippool(self, username, password, subnet_string, gateway_string, mask_string, **kwargs):  # noqa: E501
        """Post a new `/24` ip pool (subnet)  # noqa: E501

        Introduces a new `/24` ip pool that is available for assignment to FPGAs.  For now, **only /24 subnets as ip pool** are supported, so it is enough to submit the subnet base, *e.g. `10.12.200.0`*!  The submitted *gateway*, e.g. `10.12.0.1`, will be used as gateway address for all FPGAs that get an ip address out of this subnet assigned. Multiple ip pools can have the same gateway. Subnetmasks, e.g. `255.255.0.0` are used in a similar way.  Hence, the subnetmask **can not** be derived from the subnet itself. Maybe it is possible that the FPGAs of a new subnet can also talk to other subnets, so the subnet mask must be submitted separately.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_post_ippool(username, password, subnet_string, gateway_string, mask_string, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param str subnet_string: A /24 subnet (required)
        :param str gateway_string: The gateway for this subnet (required)
        :param str mask_string: The subnetmask for this subnet (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_post_ippool_with_http_info(username, password, subnet_string, gateway_string, mask_string, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_post_ippool_with_http_info(username, password, subnet_string, gateway_string, mask_string, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_post_ippool_with_http_info(self, username, password, subnet_string, gateway_string, mask_string, **kwargs):  # noqa: E501
        """Post a new `/24` ip pool (subnet)  # noqa: E501

        Introduces a new `/24` ip pool that is available for assignment to FPGAs.  For now, **only /24 subnets as ip pool** are supported, so it is enough to submit the subnet base, *e.g. `10.12.200.0`*!  The submitted *gateway*, e.g. `10.12.0.1`, will be used as gateway address for all FPGAs that get an ip address out of this subnet assigned. Multiple ip pools can have the same gateway. Subnetmasks, e.g. `255.255.0.0` are used in a similar way.  Hence, the subnetmask **can not** be derived from the subnet itself. Maybe it is possible that the FPGAs of a new subnet can also talk to other subnets, so the subnet mask must be submitted separately.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_post_ippool_with_http_info(username, password, subnet_string, gateway_string, mask_string, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param str subnet_string: A /24 subnet (required)
        :param str gateway_string: The gateway for this subnet (required)
        :param str mask_string: The subnetmask for this subnet (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'password', 'subnet_string', 'gateway_string', 'mask_string']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_post_ippool" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_post_ippool`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_post_ippool`")  # noqa: E501
        # verify the required parameter 'subnet_string' is set
        if ('subnet_string' not in params or
                params['subnet_string'] is None):
            raise ValueError("Missing the required parameter `subnet_string` when calling `cf_manager_rest_api_post_ippool`")  # noqa: E501
        # verify the required parameter 'gateway_string' is set
        if ('gateway_string' not in params or
                params['gateway_string'] is None):
            raise ValueError("Missing the required parameter `gateway_string` when calling `cf_manager_rest_api_post_ippool`")  # noqa: E501
        # verify the required parameter 'mask_string' is set
        if ('mask_string' not in params or
                params['mask_string'] is None):
            raise ValueError("Missing the required parameter `mask_string` when calling `cf_manager_rest_api_post_ippool`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501
        if 'subnet_string' in params:
            query_params.append(('subnet_string', params['subnet_string']))  # noqa: E501
        if 'gateway_string' in params:
            query_params.append(('gateway_string', params['gateway_string']))  # noqa: E501
        if 'mask_string' in params:
            query_params.append(('mask_string', params['mask_string']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/ippool/', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cf_manager_rest_api_post_mantle_logic(self, image_details, bin_file, sig_file, pr_verify_rpt, username, password, **kwargs):  # noqa: E501
        """Upload a new dynamic platform logic  # noqa: E501

        This uploads an new Image for the DYNAMIC PLATFORM LOGIC (aka FPGA PR binfile). It **must** contain the corresponding .sig file that was produced by the build.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_post_mantle_logic(image_details, bin_file, sig_file, pr_verify_rpt, username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str image_details: (required)
        :param str bin_file: (required)
        :param str sig_file: (required)
        :param str pr_verify_rpt: (required)
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_post_mantle_logic_with_http_info(image_details, bin_file, sig_file, pr_verify_rpt, username, password, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_post_mantle_logic_with_http_info(image_details, bin_file, sig_file, pr_verify_rpt, username, password, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_post_mantle_logic_with_http_info(self, image_details, bin_file, sig_file, pr_verify_rpt, username, password, **kwargs):  # noqa: E501
        """Upload a new dynamic platform logic  # noqa: E501

        This uploads an new Image for the DYNAMIC PLATFORM LOGIC (aka FPGA PR binfile). It **must** contain the corresponding .sig file that was produced by the build.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_post_mantle_logic_with_http_info(image_details, bin_file, sig_file, pr_verify_rpt, username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str image_details: (required)
        :param str bin_file: (required)
        :param str sig_file: (required)
        :param str pr_verify_rpt: (required)
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['image_details', 'bin_file', 'sig_file', 'pr_verify_rpt', 'username', 'password']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_post_mantle_logic" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'image_details' is set
        if ('image_details' not in params or
                params['image_details'] is None):
            raise ValueError("Missing the required parameter `image_details` when calling `cf_manager_rest_api_post_mantle_logic`")  # noqa: E501
        # verify the required parameter 'bin_file' is set
        if ('bin_file' not in params or
                params['bin_file'] is None):
            raise ValueError("Missing the required parameter `bin_file` when calling `cf_manager_rest_api_post_mantle_logic`")  # noqa: E501
        # verify the required parameter 'sig_file' is set
        if ('sig_file' not in params or
                params['sig_file'] is None):
            raise ValueError("Missing the required parameter `sig_file` when calling `cf_manager_rest_api_post_mantle_logic`")  # noqa: E501
        # verify the required parameter 'pr_verify_rpt' is set
        if ('pr_verify_rpt' not in params or
                params['pr_verify_rpt'] is None):
            raise ValueError("Missing the required parameter `pr_verify_rpt` when calling `cf_manager_rest_api_post_mantle_logic`")  # noqa: E501
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_post_mantle_logic`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_post_mantle_logic`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}
        if 'image_details' in params:
            form_params.append(('image_details', params['image_details']))  # noqa: E501
        if 'bin_file' in params:
            local_var_files['bin_file'] = params['bin_file']  # noqa: E501
        if 'sig_file' in params:
            local_var_files['sig_file'] = params['sig_file']  # noqa: E501
        if 'pr_verify_rpt' in params:
            local_var_files['pr_verify_rpt'] = params['pr_verify_rpt']  # noqa: E501

        body_params = None
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['multipart/form-data'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/mantle_logic', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cf_manager_rest_api_post_platform_logic(self, image_details, dcp_file, bit_file, mcs_file, sig_file, pr_verify_rpt, username, password, **kwargs):  # noqa: E501
        """Upload a new platform logic  # noqa: E501

        This uploads an new Image for the STATIC PLATFORM LOGIC (aka FPGA bitfile and flash mcsfile) and the corresponding dcp, for app / mantle builds. It also **must** contain the corresponding admin.sig file that was produced by the build.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_post_platform_logic(image_details, dcp_file, bit_file, mcs_file, sig_file, pr_verify_rpt, username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str image_details: (required)
        :param str dcp_file: (required)
        :param str bit_file: (required)
        :param str mcs_file: (required)
        :param str sig_file: (required)
        :param str pr_verify_rpt: (required)
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_post_platform_logic_with_http_info(image_details, dcp_file, bit_file, mcs_file, sig_file, pr_verify_rpt, username, password, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_post_platform_logic_with_http_info(image_details, dcp_file, bit_file, mcs_file, sig_file, pr_verify_rpt, username, password, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_post_platform_logic_with_http_info(self, image_details, dcp_file, bit_file, mcs_file, sig_file, pr_verify_rpt, username, password, **kwargs):  # noqa: E501
        """Upload a new platform logic  # noqa: E501

        This uploads an new Image for the STATIC PLATFORM LOGIC (aka FPGA bitfile and flash mcsfile) and the corresponding dcp, for app / mantle builds. It also **must** contain the corresponding admin.sig file that was produced by the build.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_post_platform_logic_with_http_info(image_details, dcp_file, bit_file, mcs_file, sig_file, pr_verify_rpt, username, password, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str image_details: (required)
        :param str dcp_file: (required)
        :param str bit_file: (required)
        :param str mcs_file: (required)
        :param str sig_file: (required)
        :param str pr_verify_rpt: (required)
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['image_details', 'dcp_file', 'bit_file', 'mcs_file', 'sig_file', 'pr_verify_rpt', 'username', 'password']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_post_platform_logic" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'image_details' is set
        if ('image_details' not in params or
                params['image_details'] is None):
            raise ValueError("Missing the required parameter `image_details` when calling `cf_manager_rest_api_post_platform_logic`")  # noqa: E501
        # verify the required parameter 'dcp_file' is set
        if ('dcp_file' not in params or
                params['dcp_file'] is None):
            raise ValueError("Missing the required parameter `dcp_file` when calling `cf_manager_rest_api_post_platform_logic`")  # noqa: E501
        # verify the required parameter 'bit_file' is set
        if ('bit_file' not in params or
                params['bit_file'] is None):
            raise ValueError("Missing the required parameter `bit_file` when calling `cf_manager_rest_api_post_platform_logic`")  # noqa: E501
        # verify the required parameter 'mcs_file' is set
        if ('mcs_file' not in params or
                params['mcs_file'] is None):
            raise ValueError("Missing the required parameter `mcs_file` when calling `cf_manager_rest_api_post_platform_logic`")  # noqa: E501
        # verify the required parameter 'sig_file' is set
        if ('sig_file' not in params or
                params['sig_file'] is None):
            raise ValueError("Missing the required parameter `sig_file` when calling `cf_manager_rest_api_post_platform_logic`")  # noqa: E501
        # verify the required parameter 'pr_verify_rpt' is set
        if ('pr_verify_rpt' not in params or
                params['pr_verify_rpt'] is None):
            raise ValueError("Missing the required parameter `pr_verify_rpt` when calling `cf_manager_rest_api_post_platform_logic`")  # noqa: E501
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_post_platform_logic`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_post_platform_logic`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}
        if 'image_details' in params:
            form_params.append(('image_details', params['image_details']))  # noqa: E501
        if 'dcp_file' in params:
            local_var_files['dcp_file'] = params['dcp_file']  # noqa: E501
        if 'bit_file' in params:
            local_var_files['bit_file'] = params['bit_file']  # noqa: E501
        if 'mcs_file' in params:
            local_var_files['mcs_file'] = params['mcs_file']  # noqa: E501
        if 'sig_file' in params:
            local_var_files['sig_file'] = params['sig_file']  # noqa: E501
        if 'pr_verify_rpt' in params:
            local_var_files['pr_verify_rpt'] = params['pr_verify_rpt']  # noqa: E501

        body_params = None
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['multipart/form-data'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/platform_logic', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cf_manager_rest_api_put_buildscript(self, username, password, bs_id, **kwargs):  # noqa: E501
        """Update a registered build script  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_put_buildscript(username, password, bs_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param int bs_id: ID of a registered build script (required)
        :param BuildScript body:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_put_buildscript_with_http_info(username, password, bs_id, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_put_buildscript_with_http_info(username, password, bs_id, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_put_buildscript_with_http_info(self, username, password, bs_id, **kwargs):  # noqa: E501
        """Update a registered build script  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_put_buildscript_with_http_info(username, password, bs_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param int bs_id: ID of a registered build script (required)
        :param BuildScript body:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'password', 'bs_id', 'body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_put_buildscript" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_put_buildscript`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_put_buildscript`")  # noqa: E501
        # verify the required parameter 'bs_id' is set
        if ('bs_id' not in params or
                params['bs_id'] is None):
            raise ValueError("Missing the required parameter `bs_id` when calling `cf_manager_rest_api_put_buildscript`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'bs_id' in params:
            path_params['bs_id'] = params['bs_id']  # noqa: E501

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/buildscripts/{bs_id}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def cf_manager_rest_api_put_composable_logic_status(self, username, password, cl_id, new_status, **kwargs):  # noqa: E501
        """Update status of a composable logic  # noqa: E501

        This call changes the status of a composable logic, to reflect the CL lifecycle (see doc).  If the status is changed to DEPRECATED or DISCHARGED, all resources are checked if some still use this CL. If yes, a warning is issued and the list of affected resources returned (response 202).  Currently, *only* the status can be changed, since all the other properties of a CL are considered static and if they should be changed, it is better to upload a new CL.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_put_composable_logic_status(username, password, cl_id, new_status, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param int cl_id: ID of a composable logic (Static Shell or Mantles) (required)
        :param str new_status: New status of the composable logic (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cf_manager_rest_api_put_composable_logic_status_with_http_info(username, password, cl_id, new_status, **kwargs)  # noqa: E501
        else:
            (data) = self.cf_manager_rest_api_put_composable_logic_status_with_http_info(username, password, cl_id, new_status, **kwargs)  # noqa: E501
            return data

    def cf_manager_rest_api_put_composable_logic_status_with_http_info(self, username, password, cl_id, new_status, **kwargs):  # noqa: E501
        """Update status of a composable logic  # noqa: E501

        This call changes the status of a composable logic, to reflect the CL lifecycle (see doc).  If the status is changed to DEPRECATED or DISCHARGED, all resources are checked if some still use this CL. If yes, a warning is issued and the list of affected resources returned (response 202).  Currently, *only* the status can be changed, since all the other properties of a CL are considered static and if they should be changed, it is better to upload a new CL.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.cf_manager_rest_api_put_composable_logic_status_with_http_info(username, password, cl_id, new_status, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str username: OpenStack username (required)
        :param str password: OpenStack password (required)
        :param int cl_id: ID of a composable logic (Static Shell or Mantles) (required)
        :param str new_status: New status of the composable logic (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['username', 'password', 'cl_id', 'new_status']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cf_manager_rest_api_put_composable_logic_status" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'username' is set
        if ('username' not in params or
                params['username'] is None):
            raise ValueError("Missing the required parameter `username` when calling `cf_manager_rest_api_put_composable_logic_status`")  # noqa: E501
        # verify the required parameter 'password' is set
        if ('password' not in params or
                params['password'] is None):
            raise ValueError("Missing the required parameter `password` when calling `cf_manager_rest_api_put_composable_logic_status`")  # noqa: E501
        # verify the required parameter 'cl_id' is set
        if ('cl_id' not in params or
                params['cl_id'] is None):
            raise ValueError("Missing the required parameter `cl_id` when calling `cf_manager_rest_api_put_composable_logic_status`")  # noqa: E501
        # verify the required parameter 'new_status' is set
        if ('new_status' not in params or
                params['new_status'] is None):
            raise ValueError("Missing the required parameter `new_status` when calling `cf_manager_rest_api_put_composable_logic_status`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'cl_id' in params:
            path_params['cl_id'] = params['cl_id']  # noqa: E501

        query_params = []
        if 'username' in params:
            query_params.append(('username', params['username']))  # noqa: E501
        if 'password' in params:
            query_params.append(('password', params['password']))  # noqa: E501
        if 'new_status' in params:
            query_params.append(('new_status', params['new_status']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/administration/composablelogic/{cl_id}/status/', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# /*******************************************************************************
#  * Copyright 2016 -- 2022 IBM Corporation
#  *
#  * Licensed under the Apache License, Version 2.0 (the "License");
#  * you may not use this file except in compliance with the License.
#  * You may obtain a copy of the License at
#  *
#  *     http://www.apache.org/licenses/LICENSE-2.0
#  *
#  * Unless required by applicable law or agreed to in writing, software
#  * distributed under the License is distributed on an "AS IS" BASIS,
#  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  * See the License for the specific language governing permissions and
#  * limitations under the License.
# *******************************************************************************/

#  *
#  *                       cloudFPGA
#  *    =============================================
#  *     Created: Apr 2019
#  *     Authors: FAB, WEI, NGL, DID
#  *
#  *     Description:
#  *       Python tool to interact with cFRM.
#  *

# @file       cFSP.py
# @brief      This is the main file of cFSP
# @author     DID
# @date       Dec 2021
# @ingroup cFSP
# @addtogroup cFSP
# \{

#! /usr/bin/env python
"""

cloudFPGA Support Package (cfsp)
cfsp is a command-line tool that helps a cloudFPGA user to work with cloudFPGA Resource Manager (cFRM).
Usage: 
    cfsp    [-c CFGFILE] [--version] [--help] [--username=<username>] [--password=<password>] [--project=<project>]
            [--image_id=<image_id>]...
            [--node_ip=<node_ip>]...
            [--node_id=<node_id>]...
            [--image_file=<image_file>] [--sig_file=<sig_file>] [--pr_verify_rpt=<pr_verify_rpt] [--dont_verify_memory=<dont_verify_memory>]
            [--limit=<limit>] [--repeat=<repeat>]
            <command> [<args>...]
    
Commands:
    user            Adding or showing the credentials of a user.
    cluster         Creating, showing and deleting clusters.
    image           Uploading, showing and deleting FPGA images.
    instance        Creating, showing and deleting instances.

Options:
    -h --help            Show this screen.
    -v --version         Show version.
    -c --config CFGFILE  Specify the configfile that rsnapshot should use 
                         [default: ./user.json]
    --username=<username>       Your ZYC2 username [default: username_example].
    --password=<password>       Your ZYC2 password [default: password_example].
    --project=<password>        The user's project [default: project_example].
    
    --image_id=<image_id>       The id of the uploaded FPGA image, or NON_FPGA for a CPU VM node [default: NON_FPGA].
    --node_ip=<node_ip>         The ip of the user's VM, e.g. a ZYC2 VM.
    --node_id=<node_id>         The id (rank) of either the VPN user's VM or FPGAs.
    
    --image_file=<image_file>      The FPGA image file to be uploaded [default: ./image.bit].
    --sig_file=<sig_file>          An FPGA bitstream signature file containing hashes from the build process.
    --pr_verify_rpt=<pr_verify_rpt An FPGA report containing the output of the automatically run pr_verify command.
    
    --dont_verify_memory=<dont_verify_memory>   If 1, don't verify the DDR4 memory during setup [default: 0].
    --limit=<limit>             The limit of get for clusters, images, instances [default: 100].
    
    --repeat=<repeat>           The numper of times to repeat the command [default: 1].
    
See 'cfsp help <command>' for more information on a specific command.

Copyright IBM Research, licensed under the Apache License 2.0.

"""

from __future__ import print_function, unicode_literals

import json
import os
import sys
from docopt import docopt
import re
from PyInquirer import prompt, print_json
from pprint import pprint
from tqdm import tqdm

import cfsp_globals
import cfsp_user
import cfsp_cluster
import cfsp_image
import cfsp_instance

from util import print_usage
import mngmt
import comm


version_path = os.path.dirname(os.path.abspath(__file__))+"/version.txt"
with open(version_path,"r") as fh:
    for line in fh:
        __version__ = line.rstrip("\n")
fh.close()

def check_credentials(CFGFILE):
    #if os.path.isfile(cfsp_globals.__cfsp_session_file__):
    #    dill.load_session(cfsp_globals.__cfsp_session_file__)
    #else:
    args = docopt(__doc__, version=__version__)
    args['<args>'] = ['load']
    cfsp_user.main(args)
    
def main():
    args = docopt(__doc__, version=__version__)
    
    argv = [args['<command>']] + args['<args>']

    for repeat_id in tqdm(range(0,int(args['--repeat']))):
        print("INFO: Repeat #"+str(repeat_id))
    
        if args['<command>'] == 'user':
            cfsp_user.main(args)
            #print(args['<args>'])
        elif args['<command>'] == 'cluster':
            check_credentials(args['--config'])
            cfsp_cluster.main(args)
        elif args['<command>'] == 'image':
            check_credentials(args['--config'])
            cfsp_image.main(args)
        elif args['<command>'] == 'instance':
            check_credentials(args['--config'])
            cfsp_instance.main(args)
        elif args['<command>'] in ['help', None]:
            if args['<args>'] == ['user']:
                print(docopt(cfsp_user.__doc__, argv=argv))
            elif args['<args>'] == ['cluster']:
                print(docopt(cfsp_cluster.__doc__, argv=argv))
            elif args['<args>'] == ['image']:
                print(docopt(cfsp_image.__doc__, argv=argv))
            elif args['<args>'] == ['instance']:
                print(docopt(cfsp_instance.__doc__, argv=argv))
            else:
                exit(print(docopt(__doc__, version=__version__)))
        else:
            print("ERROR: unknown command. Aborting...")
            exit(print(docopt(__doc__, version=__version__)))


if __name__ == '__main__':
    if not (hasattr(sys, 'real_prefix') or (hasattr(sys, 'base_prefix') and sys.base_prefix != sys.prefix)):
        # This works with virtualenv for Python 3 and 2 and also for the venv module in Python 3
        print("ERROR: It looks like this cFSP isn't running in a virtual environment. Aborting.")
        sys.exit(1)
        
    
    main()
    exit(0)



# ! \} 



# /*******************************************************************************
#  * Copyright 2016 -- 2022 IBM Corporation
#  *
#  * Licensed under the Apache License, Version 2.0 (the "License");
#  * you may not use this file except in compliance with the License.
#  * You may obtain a copy of the License at
#  *
#  *     http://www.apache.org/licenses/LICENSE-2.0
#  *
#  * Unless required by applicable law or agreed to in writing, software
#  * distributed under the License is distributed on an "AS IS" BASIS,
#  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  * See the License for the specific language governing permissions and
#  * limitations under the License.
# *******************************************************************************/

#  *
#  *                       cloudFPGA
#  *     Copyright IBM Research, All Rights Reserved
#  *    =============================================
#  *     Created: Sep. 2020
#  *     Authors: FAB, WEI, NGL, DID
#  *
#  *     Description:
#  *      A Python library with functions for accessing cFRM and create/delete images, instances, clusters etc.
#  *
#  *

# assemble the package
from .util  import print_usage
from . import mngmt
from . import comm

